#!/usr/bin/python

import re
import string
import sys


STAG = '<location>'
ETAG = '</location>'
TAG = 'LOCATION'

tokens = []
classes = []
linebreaks = []

regex = {
   # normalize AM/PM
   re.compile('a\.m\.?') : ' AM ',
   re.compile('p\.m\.?') : ' PM ',
   # only separate period followed by space or end of line
   re.compile('\.( |$)') : ' . ',
   # separate commas, dashes
   re.compile(',') : ' , ',
   re.compile('-') : ' - ',
   # separate parentheses
   re.compile('\(') : ' ( ',
   re.compile('\)') : ' ) ',
   # make sure we tokenize before start tags and after end tags,
   # and that they are flush against a token if possible by removing whitespace
   re.compile(re.escape(STAG) + '\s*') : ' ' + STAG,
   re.compile('\s*?' + re.escape(ETAG)) : ETAG + ' ',
}

re_tag = re.compile('<.*?>')


def main():
   for line in sys.stdin:
      line = line.rstrip()

      # Normalize with regexes
      for r in regex.keys():
         line = r.sub(regex[r], line)

      # skip it if it is only tags (but not if it has location or /location)
      if (not STAG in ''.join(line.split('/')) and
          re_tag.sub('', line).strip() == ''):
          continue


      inside = False
      for word in line.split():
         if word.startswith(STAG) and word.endswith(ETAG):
            """ Using label "W-STIME" for "Whole",
                separating one token entities from
                multi-token entities """
            classes.append('W-' + TAG)
         elif word.startswith(STAG):
            classes.append('B-' + TAG)
            inside = True
         elif word.endswith(ETAG):
            classes.append('E-' + TAG)
            inside = False
         elif inside:
            classes.append('I-' + TAG)
         else:
            classes.append('O')

         # delete tags
         word = re_tag.sub('', word)

         if word.strip() == '':
            tokens.append('NULL')
         else: tokens.append(word)

         linebreaks.append(0)

      """ record a feature for the end of each line (value=1). If we have
          a following blank line, increment it to 2 (but stop
          incrementing after that) """
      if linebreaks[len(linebreaks) -1] <2:
         linebreaks[len(linebreaks) -1] += 1;

   # EOF counts as a double blank line
   linebreaks[len(linebreaks)-1] = 2


   for i, (tok, cl) in enumerate(zip(tokens, classes)):
      # Print features and label

      #TODO Add features here

      # Feature 1 is current token
      print(tok, end=' ')

      print('upper' + tok[0] if tok[0].isupper() else 'NO' + tok[0], end= ' ')
      print('punct' + tok[0] if tok[0] in string.punctuation else 'NO' + tok[0], end= ' ')


      # Label
      print(cl,end='\n')

      # Sequence splitting: we have a new sequence if we have a period
      if tok == '.':
         print()


if __name__ == "__main__": main()
